@@ -6,10 +6,10 @@ use Config::Auto;
 use Cwd;
 use warnings;
 use strict;
-our $VERSION = '2.1.0';
+our $VERSION = '2.2.4';
 
 # Version of dpkg-cross environment.
-my $DPKGCROSSVERSION = "2.1.0";
+my $DPKGCROSSVERSION = "2.2.4";
 
 require Exporter;
 
@@ -17,15 +17,15 @@ use vars qw (@ISA @EXPORT @EXPORT_OK $conffile $private_conffile
 $progname %archtable %archdetecttable %crossprefixtable
 %std_tools %pkgvars %allcrossroots $arch $default_arch
 $deb_host_gnu_type $crossbase $crossprefix $crossdir $crossbin
-$crosslib $crossroot $crossinc $crosslib64 $package $mode $tool_
+$crosslib $crossroot $crossinc $crosslib64 $crosslib32 $package $mode $tool_
 %config @keepdeps %allcrossroots @removedeps $maintainer
 $compilerpath %debug_data);
 @ISA       = qw(Exporter);
 @EXPORT    = qw( read_config setup get_architecture create_tmpdir
 convert_path get_config get_version rewrite_pkg_name dump_debug_data
-check_arch convert_filename);
+check_arch convert_filename detect_arch get_endianness);
 @EXPORT_OK = qw(dpkgcross_application simplify_path setup_cross_env
-detect_arch get_tool convert_ld_library_path get_keepdeps get_removedeps);
+get_tool convert_ld_library_path get_keepdeps get_removedeps );
 
 =pod
 
@@ -55,7 +55,7 @@ Copyright (C) 2004  Raphael Bossek <bossekr@debian.org>
 
 =item *
 
-Copyright (c) 2007  Neil Williams <codehelp@debian.org>
+Copyright (c) 2007-2008  Neil Williams <codehelp@debian.org>
 =back
 
  This program is free software; you can redistribute it and/or modify
@@ -111,7 +111,7 @@ $progname = basename($0);
 );
 
 # Regexps to parse 'file' output to detect arch of ELF binary
-# Note that it is not always possibe to restore Debian architecture
+# Note that it is not always possible to restore Debian architecture
 # name by binary: e.g. i386-linux and i386-hurd use same ELF shared
 # library format.
 # Special arch name 'AR' is used to detect ar archives that need
@@ -125,6 +125,7 @@ $progname = basename($0);
 	'm68k' => 'ELF 32-bit MSB .* 680[02]0',
 	'arm' => 'ELF 32-bit LSB .* ARM',
 	'armeb' => 'ELF 32-bit MSB .* ARM',
+	'armel' => 'ELF 32-bit LSB .* SYSV',
 	'powerpc' => 'ELF 32-bit MSB .* PowerPC',
 	'powerpc64' => 'ELF 64-bit MSB .* PowerPC',
 	'mips' => 'ELF 32-bit MSB .* MIPS',
@@ -223,6 +224,7 @@ sub read_config {
 	$crossbin = $conf->{'crossbin'};
 	$crosslib = $conf->{'crosslib'};
 	$crosslib64 = $conf->{'crosslib64'};
+	$crosslib32 = $conf->{'crosslib32'};
 	$crossinc = $conf->{'crossinc'};
 	$maintainer = $conf->{'maintainer'};
 	$compilerpath = $conf->{'compilerpath'};
@@ -251,6 +253,7 @@ sub read_config {
 		$crossbin ||= $conf->{'crossbin'};
 		$crosslib ||= $conf->{'crosslib'};
 		$crosslib64 ||= $conf->{'crosslib64'};
+		$crosslib32 ||= $conf->{'crosslib32'};
 		$crossinc ||= $conf->{'crossinc'};
 		$maintainer ||= $conf->{'maintainer'};
 		$compilerpath ||= $conf->{'compilerpath'};
@@ -394,7 +397,7 @@ sub check_arch($) {
 
 =head1 setup
 
-Set global variables '$arch', '$crossbase', '$crossbin',
+Set global variables '$arch', '$crossbase', '$crossbin', '$crosslib32',
 '$crossdir', '$crossinc', '$crosslib', '$crosslib64', '$crossprefix',
 '$compilerpath' and '$deb_host_gnu_type' to defaults and substitute
 them with variables from '%conf' and '$arch'.
@@ -430,12 +433,14 @@ sub setup() {
 	}
 
 	$crosslib64 ||= $crosslib . "64";
+	$crosslib32 ||= $crosslib . "32";
 	$config{'crossbase'} = $crossbase;
 	$config{'crossprefix'} = $crossprefix;
 	$config{'crossdir'} = $crossdir;
 	$config{'crossbin'} = $crossbin;
 	$config{'crosslib'} = $crosslib;
 	$config{'crosslib64'} = $crosslib64;
+	$config{'crosslib32'} = $crosslib32;
 	$config{'crossinc'} = $crossinc;
 	$config{'crossroot'} = $crossroot;
 
@@ -455,6 +460,7 @@ sub setup() {
 	$crossbin = $config{'crossbin'};
 	$crosslib = $config{'crosslib'};
 	$crosslib64 = $config{'crosslib64'};
+	$crosslib32 = $config{'crosslib32'};
 	$crossinc = $config{'crossinc'};
 }
 
@@ -485,7 +491,7 @@ sub create_tmpdir ($) {
 =head1 convert_path($path)
 
 Convert path, substituting '$crossinc', '$crosslib', '$crosslib64',
-'$crossdir'. This function will be used while building foreign
+'$crosslib32', '$crossdir'. This function will be used while building foreign
 binary packages or converting GCC options.
 
  $1: Directory (and file) to convert.
@@ -502,6 +508,8 @@ sub convert_path ($) {
 		$path = "$crosslib/$'";
 	} elsif ($path =~ /^(\/usr(\/X11R6)?)?\/lib64\//) {
 		$path = "$crosslib64/$'";
+	} elsif ($path =~ /^(\/usr(\/X11R6)?)?\/lib32\//) {
+		$path = "$crosslib32/$'";
 	} elsif ($path =~ /^\/usr\/\w+-\w+(-\w+(-\w+)?)?\//) {
 		# leave alone
 	} else {
@@ -550,51 +558,31 @@ sub simplify_path ($) {
 	return $path;
 }
 
-=head1 Legacy code
+=head1 get_endianness
 
-The following functions were part of dpkg-cross.pl <= 1.39
-and were only used by the dpkg-cross diversions of dpkg-buildpackage
-or dpkg-shlibdeps. The functions and the scripts are retained for
-now as legacy code or for bespoke implementations but may be removed
-at a later date. It is NOT recommended to use these functions for
-newly written code. Scripts that do use these functions must import
-them explicitly.
+Provide a central function to query the endianness of the
+current cross building architecture.
 
-=head1 get_keepdeps
+Parses /etc/dpkg-cross/cross-config.$arch to convert the
+autotools cache value into a general purpose string.
 
-Array containing the list of dependencies to be kept when building a
-cross package. Read from the $conffile.
-
-Largely superceded by the -X support in dpkg-cross itself.
-
- Deprecated: May be removed in future versions.
+Returns 'big' or 'little' or undefined on error.
 
 =cut
 
-sub get_keepdeps {
-	return @keepdeps;
-}
-
-=head1 get_keepdeps
-
-Array containing the list of dependencies to be remove when building a
-cross package. Read from the $conffile.
-
-Largely superceded by the -X support in dpkg-cross itself.
-
- Deprecated: May be removed in future versions.
-
-=cut
-
-sub get_removedeps {
-	return @removedeps;
+sub get_endianness
+{
+	my $cfile = "/etc/dpkg-cross/cross-config.$arch";
+	my $endian;
+	return $endian if (! -f $cfile);
+	my $config = Config::Auto::parse("$cfile", format => "equal");
+	return $endian if (not defined ($config->{'ac_cv_c_bigendian'}));
+	$endian = ($config->{'ac_cv_c_bigendian'} eq "yes") ? "big" : "little";
+	return $endian;
 }
 
 =head2 detect_arch
 
-Legacy code - only used by the old version of dpkg-shlibdeps,
-objcopy, objdump and strip from dpkg-cross <= 1.39
-
 Detect architecture of a given ELF or AR file using 'file' output.
 In general case it seems to be impossible to distinguish between
 OSes (e.g. between i386-linux and i386-hurd), so just detect
@@ -604,6 +592,9 @@ CPU architecture
 
 return: Detected architecture name on success, empty string on failure
 
+Note that the table used by this routine is incomplete and may not
+always identify the arch. Always check the return value.
+
 =cut
 
 sub detect_arch {
@@ -637,6 +628,46 @@ sub detect_arch {
 	return "";
 }
 
+=head1 Legacy code
+
+The following functions were part of dpkg-cross.pl <= 1.39
+and were only used by the dpkg-cross diversions of dpkg-buildpackage
+or dpkg-shlibdeps. The functions and the scripts are retained for
+now as legacy code or for bespoke implementations but may be removed
+at a later date. It is NOT recommended to use these functions for
+newly written code. Scripts that do use these functions must import
+them explicitly.
+
+=head1 get_keepdeps
+
+Array containing the list of dependencies to be kept when building a
+cross package. Read from the $conffile.
+
+Largely superceded by the -X support in dpkg-cross itself.
+
+ Deprecated: May be removed in future versions.
+
+=cut
+
+sub get_keepdeps {
+	return @keepdeps;
+}
+
+=head1 get_removedeps
+
+Array containing the list of dependencies to be remove when building a
+cross package. Read from the $conffile.
+
+Largely superceded by the -X support in dpkg-cross itself.
+
+ Deprecated: May be removed in future versions.
+
+=cut
+
+sub get_removedeps {
+	return @removedeps;
+}
+
 ################################################################################
 # read_crosstools_file(): DEPRECATED
 #     Reads /etc/dpkg-cross/crosstools or ~/.dpkg-cross/crosstools and saves results
